
#############################################################################
##
## Name [i,j]     
## Numb  OFFSET_VARS + i(i-1)/2 + j
## Indet (q-j,i)
## Value Gcd(q-j,i)
##
## Assume: i > 1, 0 <= j < i
##
OFFSET_VARS := 1000;

NameToNumb := function( name )
    local i, j;
    if name = "q" then return OFFSET_VARS; fi;
    i := name[1]; j := name[2];
    if i <= 1 then return fail; fi;
    return OFFSET_VARS + i*(i-1)/2 + (j mod i);
end;

NumbToName := function( n )
    local i, j;

    # get rid of overhead and check
    j := n - OFFSET_VARS;
    if j = 0 then return "q"; fi;
    if j < 0 then return false; fi;

    # now have i*(i-1)/2 + j
    i := 2; while (j > i) do j := j-i; i := i+1; od;
    return [i, j-1];
end;

NameToIndet := function( name )
    local v, s, i, j;

    # 1. case
    if name = "q" then 
        v := Indeterminate(Rationals, OFFSET_VARS); 
        if not HasName(v) then SetName(v,"q"); fi;
    fi;
  
    # 2. case
    if Length(name)=2 then 
        v := Indeterminate(Rationals, NameToNumb(name));
        if not HasName(v) then 
            i := name[1]; j := (name[2] mod i);
            s := Concatenation("(q-",String(j),",",String(i),")");
            SetName(v,s); 
        fi;
    fi;

    return v;
end;

IndetToName := function( var )
    local n;
    n := ExtRepPolynomialRatFun(var)[1][1];
    if n = OFFSET_VARS then return "q"; fi;
    if n <= OFFSET_VARS then return false; fi;
    return NumbToName(n);
end;

ValueOfPorcPoly := function( poly, q )
    local e, v, i, m, w, j, a, k;

    e := ExtRepPolynomialRatFun(poly);
    v := 0;
    for i in [1,3..Length(e)-1] do
        m := e[i];
        w := e[i+1];
        for j in [1,3..Length(m)-1] do
            a := NumbToName(m[j]);
            if a = "q" then 
                w := w * q^m[j+1];
            else
                w := w * Gcd(q-a[2],a[1])^m[j+1];
            fi;
        od;
        v := v + w;
    od;
    return v;
end;

ValueOfPorcRatFun := function( poly, q )
    return ValueOfPorcPoly(NumeratorOfRationalFunction(poly),q)/
           ValueOfPorcPoly(DenominatorOfRationalFunction(poly),q);
end;

DegreeOfPorcPoly := function( poly )
    if poly = 0*poly then return -infinity; fi;
    return DegreeIndeterminate(poly, NameToIndet("q"));
end;

IndetsOfPorcPoly := function(poly)
    local e, v, i, j;
    e := ExtRepPolynomialRatFun(poly);
    v := [];
    for i in [1,3..Length(e)-1] do
        for j in [1,3..Length(e[i])-1] do
            AddSet(v, e[i][j]);
        od;
    od;
    return List(v, x-> Indeterminate(Rationals, x));
end;

PrintPorcPoly := function(poly)
    local c, i;
    c := PolynomialCoefficientsOfPolynomial( poly, NameToIndet("q") );
    for i in [1..Length(c)] do
        if c[i] <> 0*c[i] then 
            if c[i] = c[i]^0 then 
                Print("q^",i-1);
            else
                Print("(",c[i],")*q^",i-1);
            fi;
            if i < Length(c) then Print(" + "); fi;
        fi;
    od;
    Print("\n");
end;

PrintPorcPolyInLatexFormat := function( porc, file )
    local coeff, l, c, d, i;
    
    coeff := PolynomialCoefficientsOfPolynomial( porc, NameToIndet("q") );
    l := Length(coeff);

    PrintTo(file, "\\begin{eqnarray*} \n");
    for i in Reversed([1..l]) do
        c := coeff[i];
        d := i-1;
        if c <> 0*c then 
            AppendTo(file, "&+& q^{",d,"}");
            if IsConstantRationalFunction(c) and c <> c^0 then 
                AppendTo(file, " \\cdot ", c );
            elif c <> c^0 then 
                AppendTo(file, "\\cdot (");
                AppendTo(file, c );
                AppendTo(file, ")");
            fi;
            AppendTo(file, " \\\\ \n");
        fi;
    od;
    AppendTo(file, "\\end{eqnarray*}");

end;
    

